@@ -6,7 +6,9 @@ use crate::{Message, Source};
 /// Callbag operator that flattens a higher-order callbag source.
 ///
 /// Like RxJS "switch" or xstream "flatten". Use it with map to get behavior equivalent to
-/// "switchMap". Works on either pullable or listenable sources.
+/// "switchMap".
+///
+/// Works on either pullable or listenable sources.
 ///
 /// See <https://github.com/staltz/callbag-flatten/blob/9d08c8807802243517697dd7401a9d5d2ba69c24/index.js#L1-L43>
 pub fn flatten<T: 'static>(source: Source<Source<T>>) -> Source<T> {
@@ -29,9 +31,7 @@ pub fn flatten<T: 'static>(source: Source<Source<T>>) -> Source<T> {
                     Message::Pull => {
                         if let Some(inner_talkback) = &*inner_talkback.load() {
                             inner_talkback(Message::Pull);
-                            return;
-                        }
-                        if let Some(outer_talkback) = &*outer_talkback.load() {
+                        } else if let Some(outer_talkback) = &*outer_talkback.load() {
                             outer_talkback(Message::Pull);
                         }
                     }
@@ -82,12 +82,12 @@ pub fn flatten<T: 'static>(source: Source<Source<T>>) -> Source<T> {
                                     Message::Terminate => {
                                         if outer_talkback.load().is_none() {
                                             sink(Message::Terminate);
-                                            return;
+                                        } else {
+                                            inner_talkback.store(None);
+                                            let outer_talkback = outer_talkback.load();
+                                            let outer_talkback = outer_talkback.as_ref().unwrap();
+                                            outer_talkback(Message::Pull);
                                         }
-                                        inner_talkback.store(None);
-                                        let outer_talkback = outer_talkback.load();
-                                        let outer_talkback = outer_talkback.as_ref().unwrap();
-                                        outer_talkback(Message::Pull);
                                     }
                                 })
                                 .into(),
@@ -105,9 +105,9 @@ pub fn flatten<T: 'static>(source: Source<Source<T>>) -> Source<T> {
                         Message::Terminate => {
                             if inner_talkback.load().is_none() {
                                 sink(Message::Terminate);
-                                return;
+                            } else {
+                                outer_talkback.store(None);
                             }
-                            outer_talkback.store(None);
                         }
                     }
                 })
