@@ -1,11 +1,14 @@
-use std::sync::{Arc, RwLock};
+use arc_swap::ArcSwap;
+use std::sync::Arc;
 
 use crate::{Message, Source};
 
 /// Callbag operator that combines consecutive values from the same source.
 ///
 /// It's essentially like array `.reduce`, but delivers a new accumulated value for each value from
-/// the callbag source. Works on either pullable or listenable sources.
+/// the callbag source.
+///
+/// Works on either pullable or listenable sources.
 ///
 /// See <https://github.com/staltz/callbag-scan/blob/4ade1071e52f53a4b712d38f4e975f52ce8710c8/readme.js#L28-L40>
 pub fn scan<I: 'static, O: 'static, F: 'static>(
@@ -22,7 +25,7 @@ where
             let seed = seed.clone();
             move |message| {
                 if let Message::Handshake(sink) = message {
-                    let acc = Arc::new(RwLock::new(seed.clone()));
+                    let acc = ArcSwap::from_pointee(seed.clone());
                     source(Message::Handshake(
                         ({
                             let reducer = reducer.clone();
@@ -50,15 +53,8 @@ where
                                     ));
                                 }
                                 Message::Data(data) => {
-                                    {
-                                        let mut acc = acc.write().unwrap();
-                                        *acc = reducer(acc.clone(), data);
-                                    }
-                                    let acc = {
-                                        let acc = &*acc.read().unwrap();
-                                        acc.clone()
-                                    };
-                                    sink(Message::Data(acc));
+                                    acc.store(Arc::new(reducer((**acc.load()).clone(), data)));
+                                    sink(Message::Data((**acc.load()).clone()));
                                 }
                                 Message::Pull => {
                                     panic!("source must not pull");
